
package interfazdiccionario;

import Modelo.FuenteDeDatos;
import Otros.Diccionario;
import java.util.ArrayList;
import maquinaEstados.maquinaEstados;

/**
 * Controlador de la vista de creacion del diccionario
 * @author Raúl Hernansanz Quevedo
 */
public class ControladorDiccionario {
    private VistaDiccionario mivista;
    
    private static Diccionario dic = new Diccionario(new ArrayList(),new ArrayList(),new ArrayList());
    
    /**
     * Constructor del controlador de la vista de creacion de diccionario
     * @param mivista vista de la ventana de creacion del diccionario
     */
    public ControladorDiccionario(VistaDiccionario mivista){
        this.mivista = mivista;  
    }
    
    
    /**
     * Limpia el diccionario local y le pide a la maquina de estados volver a la ventana 
     * principal
     */
    public static void procesaCancelar(){
        dic = new Diccionario(new ArrayList(),new ArrayList(),new ArrayList());
        maquinaEstados.interfaz();
    }
    
    /**
     * Comprueba errores y añade la nueva variable al diccionario local, actualizando
     * la vista con ella
     */
    public void procesaAñadir(){
        boolean error = false;
        String nombre = mivista.getNombre();
        String codigo = mivista.getCodigo();
        String descripcion = mivista.getDescripcion();
        if(nombre.length()<3){
            mivista.errorNombre();
            error = true;
        }else{
            if(codigo.equals("")|| codigo.equals(" ")){
                mivista.errorCodigo();
                error = true;
            }else{
                if(descripcion.length()<3){
                    mivista.errorDescripcion();
                    error = true;
                }           
            }
        }
        if(!error){
            this.dic.getNombres().add(nombre);
            this.dic.getCodigos().add(codigo);
            this.dic.getDescripciones().add(descripcion);
            mivista.nuevoRegistro();
            mivista.actualizaDiccionario(dic.getNombres(),dic.getCodigos(),dic.getDescripciones());
        }
    }
    
    /**
     * Pide a la vista que limpie el campo del nombre de la variable
     */
    public void limpiaNombre(){
        mivista.limpiaNombre();
    }
    
    /**
     * Pide a la vista que limpie el campo del codigo
     */
    public void limpiaCodigo(){
        String codigo = mivista.getCodigo();
        if(codigo.equals("")||codigo.equals(" ")||codigo.equals("Error")){
            mivista.limpiaCodigo();
        }
    }
    
    /**
     * Pide a la vista que limpie el campo de la descripcion
     */
    public void limpiaDescripcion(){
        String descripcion = mivista.getDescripcion();
        if(descripcion.length()<3||descripcion.equals("Error")){
            mivista.limpiaDescripcion();
        }
    }
    
    /**
     * Actualiza el diccionario de la fuente de datos, pone a true el campo
     * de existencia de diccionario y le pide a la maquina de estados volver
     * a la ventana principal
     */
    public void procesaAceptar(){
        if(!dic.getNombres().isEmpty()){
           FuenteDeDatos.setDiccionario(dic); 
           FuenteDeDatos.setExisteDTrue();
           maquinaEstados.interfaz();
        }else{
           FuenteDeDatos.setDiccionario(new Diccionario());
           FuenteDeDatos.setExisteDFalse();
           maquinaEstados.interfaz(); 
        }
    }
    
    /**
     * Actualiza el diccionario local con las variables que ya estuvieran en el diccionario
     * de la fuente de datos y le pide a la vista que se actualice con ellas
     */
    public void actualizar(){
        ArrayList<String> nombres = FuenteDeDatos.getDiccionario().getNombres();
        ArrayList<String> codigos = FuenteDeDatos.getDiccionario().getCodigos();
        ArrayList<String> descripciones = FuenteDeDatos.getDiccionario().getDescripciones();
        dic = new Diccionario();
        
        for(int i = 0 ; i<nombres.size();i++){
            dic.añadeNombre(nombres.get(i));
            dic.añadeCodigo(codigos.get(i));
            dic.añadeDescripcion(descripciones.get(i));
        }
        
        this.mivista.actualizaDiccionario(dic.getNombres(),dic.getCodigos(),dic.getDescripciones());
        
    }
    
    /**
     * Elimina una variable de la lista en funcion de la posicion relativa del raton
     * @param x posicion x del raton
     * @param y posicion y del raton
     */
    public void eliminaPorPos(int x, int y){
        int indice = obtenerIndice(y);
        if(!dic.getNombres().isEmpty()){
            if(dic.getNombres().size()>=indice){
                dic.getNombres().remove(indice);
                dic.getCodigos().remove(indice);
                dic.getDescripciones().remove(indice);
                mivista.actualizaDiccionario(dic.getNombres(),dic.getCodigos(),dic.getDescripciones()); 
            }
        }
    }
    
    
    /**
     * Obtiene el indice de la variable a eliminar en la lista en funion de la coordenada
     * y de la posicion del raton
     * @param y posicion y del raton
     * @return indice de la variable a eliminar
     */
    public int obtenerIndice(int y){
        int indice=0;
        boolean bandera = false;
        if(y>55){
            int ref = 55;
            while(!bandera){
                if(y<=ref){
                   bandera = true;
                }else{
                    indice += 1;
                    ref += 44;
                }
            }

        }
        return indice;        
    }
    
    
    
}


