/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package interfazusuario;

import Modelo.FuenteDeDatos;
import Otros.Diccionario;
import Otros.Visualizacion;
import java.awt.Component;
import java.awt.FileDialog;
import java.util.ArrayList;
import maquinaEstados.maquinaEstados;
import java.io.*;
import javax.swing.JFileChooser;

/**
 *Controlador de la ventana principal de la interfaz
 * @author Raúl Hernansanz Quevedo
 */
public class ControladorInterfaz {
    private VistaInterfaz mivista;
    private static String rutaCompleta = null;

    /**
     * Constructor del controlador de la vista principal de la interfaz
     * @param mivista vista de la ventana principal
     */
    public ControladorInterfaz(VistaInterfaz mivista){
        this.mivista = mivista;  
    }
    
    /**
     * Obtiene el separador de la vista y se la pasa a la fuente de datos
     */
    public void procesaSeparador(){
        String separador = mivista.getSeparador();
        FuenteDeDatos.setSeparador(separador);
    }
    
    /**
     * Obtiene el formato de destino de la vista y se la pasa a la fuente de datos
     */
    public void procesaFormatoDestino(){
        String formatoDestino = mivista.getFormatoDestino();
        FuenteDeDatos.setformatoDestino(formatoDestino);
    }
    
    /**
     * Le pide a la máquina de estados que cambie a la ventana del diccionario
     */
    public void procesaCreaDiccionario(){
        maquinaEstados.diccionario();
    }
    
    /**
     * Actualiza la vista en función de la visualizacion que este en el combo box
     * @param visualizacion visualizacion seleccionada actualmente
     */
    public void  actualizar(String visualizacion){
        if(visualizacion.equals("Grafico de barras") || visualizacion.equals("Grafico de sectores") || visualizacion.equals("Grafico de líneas")){
           String [] medidas = new String [3];
           medidas[0] = "Media";
           medidas[1] = "Total";
           medidas[2] = "Porcentaje";
           mivista.setMedidas(medidas); 
        } else {
            String [] medidas = new String [3];
            medidas[0] = "Media";
            medidas[1] = "Total";
            medidas[2] = "Porcentaje";
            mivista.setMedidas(medidas);    
        }
        if(visualizacion.equals("Grafico de sectores")){
            mivista.deshabilitaCruce();
        }else{
            mivista.habilitaCruce();
        }
        
        if(!FuenteDeDatos.getDiccionario().getNombres().isEmpty()){
            mivista.hayDiccionario();
        }
    }
    
    /**
     * Abre el explorador de archivos para poder seleccionar el archivo de origen
     * y actualiza la vista y la fuente de datos con la ruta
     * @param componente componente recibido de la vista
     */
    public void procesaRutaOrigen(Component componente){
        JFileChooser dig = new JFileChooser();
        int option = dig.showOpenDialog(componente);
        if(option == JFileChooser.APPROVE_OPTION){
            String file = dig.getSelectedFile().getPath();
            mivista.setRutaOrigen(file);
            FuenteDeDatos.setrutaOrigen(file);
        }
    }
    
    /**
     * Abre el explorador de archivos para poder seleccionar la ruta de destino
     * y actualiza la vista y la fuente de datos con ella
     * @param componente componente recibido de la vista
     */
    public void procesaRutaDestino(Component componente){
        JFileChooser dig = new JFileChooser();
        dig.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        int option = dig.showOpenDialog(componente);
        if(option == JFileChooser.APPROVE_OPTION){
            String file = dig.getSelectedFile().getPath();
            mivista.setRutaDestino(file);
            FuenteDeDatos.setrutaDestino(file);
        }
    }
    
    /**
     * Abre el explorador de archivos para poder seleccionar la ruta de R
     * y actualiza la vista y la fuente de datos con ella
     * @param componente componente recibido de la vista
     */
    public void procesaRutaR(Component componente){
        JFileChooser dig = new JFileChooser();
        int option = dig.showOpenDialog(componente);
        if(option == JFileChooser.APPROVE_OPTION){
            String file = dig.getSelectedFile().getPath();
            mivista.setRutaR(file);
            FuenteDeDatos.setRutaR(file);
        }
    }
    
    /**
     * Metodo que añade las visualizaciones que desee el usuario a la fuente de datos
     * y actualiza la vista con ellas
     */
    public void procesaAñadir(){
        
        boolean error = false;
        String tipo = mivista.getTipoVisualizacion();
        String medida = mivista.getMedida();
        String vrespuesta = mivista.getVrespuesta();
        String vcruce1 = mivista.getVcruce1();
        String vcruce2 = mivista.getVcruce2();
        if(vcruce2.equals("Opcional")|| vcruce2.equals("")){
            vcruce2="-";
        }
        if(vrespuesta.equals("")){
            mivista.errorRespuesta();
            error = true;
        }
        if(vcruce1.equals("")){
            mivista.errorCruce1();
            error = true;
        }
        if (!error){
            
            Visualizacion v = new Visualizacion (tipo, medida, vrespuesta, vcruce1, vcruce2);
        
            maquinaEstados.getModelo().añadeVisualizaciones(v);
            ArrayList<Visualizacion> visActuales = maquinaEstados.getModelo().getVisualizaciones();
            mivista.actualizaVisualizaciones(visActuales);
        }
    }   
    
    /**
     * Metodo que le pide a la vista que limpie el campo de la segunda variable de cruce
     */
    public void limpiaCruce2(){
        if(mivista.getVcruce2().equals("Opcional")){
            mivista.limpiaCruce2();
        }
    }
    
    /**
     * Metodo que le pide a la vista que limpie el campo de la variable respuesta
     */
    public void limpiaRespuesta(){
        if(mivista.getVrespuesta().equals("Falta la variable respuesta")){
            mivista.limpiaRespuesta();
        }
    }
    
    /**
     * Metodo que le pide a la vista que limpie el campo de la primera variable de cruce
     */
    public void limpiaCruce1(){
        if(mivista.getVcruce1().equals("Falta la variable de cruce")){
            mivista.limpiaCruce1();
        }
    }
    
    /**
     * Metodo que le pide a la vista que limpie el campo del factor de elevación
     */
    public void limpiaElevacion(){
        if(mivista.getElevacion().equals("Introduce el factor de elevacion")){
            mivista.limpiaElevacion();
        }
    }
    
    /**
     * Metodo que elimina la visualizacion que indique usuario haciendo doble click
     * @param x posicion x del raton relativa al cuadro de texto de las visualizaciones
     * @param y posicion y del raton relativa al cuadro de texto de las visualizaciones
     */
    public void eliminaPorPos(int x, int y){
        ArrayList<Visualizacion> visActuales = maquinaEstados.getModelo().getVisualizaciones();       
        if (!visActuales.isEmpty()){
            int indice = obtenerIndice(y);
            if(visActuales.size()>=indice){
                maquinaEstados.getModelo().eliminaVisualizaciones(indice);
                visActuales = maquinaEstados.getModelo().getVisualizaciones();
                mivista.actualizaVisualizaciones(visActuales); 
            }   
        }
    }
    
    /**
     * Metodo que, en funcion de la posicion en el eje y del raton, devuelve el
     * índice de la visualizacion a la que corresponde dicha posicion en el array
     * de visualizaciones
     * @param y coordenada y
     * @return indice de la visualizacion que se desea eliminar
     */
    public int obtenerIndice(int y){
        int indice=0;
        boolean bandera = false;
        if(y>134){
            int ref = 134;
            while(!bandera){
                if(y<=ref){
                   bandera = true;
                }else{
                    indice += 1;
                    ref += 126;
                }
            }

        }
        return indice;        
    }
    
    
    /**
     * Metodo que lee el fichero de configuracion de la interfaz y actualiza la vista
     * @throws FileNotFoundException excepcion de fichero no encontrado
     * @throws IOException excepcion de entrada/salida
     */
    public void leefichero() throws FileNotFoundException, IOException{
        File archivo = new File (".\\parametros.txt"); //HAY QUE PONER RUTA RELATIVA
        FileReader fr = new FileReader (archivo);
        BufferedReader br = new BufferedReader(fr);
        
        String formatos = br.readLine();
        String[] Aformatos = formatos.split(",");
        String separadores = br.readLine();
        String[] Aseparadores = separadores.split(",");
        String tipos = br.readLine();
        String[] Atipos = tipos.split(",");
        mivista.componentesFichero(Aformatos,Aseparadores,Atipos);
    }
    
    /**
     * Metodo que actualiza la vista con todos los parámetros que ya esten definidos
     * en la fuente de datos
     */
    public void actualizate(){
        ArrayList<Visualizacion> visActuales = maquinaEstados.getModelo().getVisualizaciones();
        if (!visActuales.isEmpty()){
            mivista.actualizaVisualizaciones(visActuales);  
        }
        if(maquinaEstados.getModelo().getrutaOrigen() != null){
            mivista.setRutaOrigen(maquinaEstados.getModelo().getrutaOrigen()); 
        }
        if(!maquinaEstados.getModelo().getrutaDestino().equals(".")){
            mivista.setRutaDestino(maquinaEstados.getModelo().getrutaDestino());
        }
        if(maquinaEstados.getModelo().getRutaR() != null){
            mivista.setRutaR(maquinaEstados.getModelo().getRutaR());
        }
        if(maquinaEstados.getModelo().getNombreD() != null && !maquinaEstados.getModelo().getNombreD().equals("informe")){
            mivista.setNombreD(maquinaEstados.getModelo().getNombreD());
        }
        System.out.println(maquinaEstados.getModelo().getNombreD());
        mivista.setSeparador(maquinaEstados.getModelo().getSeparador());
        mivista.setDestino(maquinaEstados.getModelo().getformatoDestino());
        mivista.setElevacion(maquinaEstados.getModelo().getElevacion());
       
    }

    /**
     * Metodo que comprueba posibles errores y crea los dos ficheros de texto resultado
     * y, con ellos creados, llama al programa de R. En el caso de que encuentre que el fichero
     * que el usuario desea crear ya existe, llama a la maquina de estados para que 
     * muestre la ventana de alerta
     * @throws IOException excepcion de entrada/salida
     */
    public void procesaAceptar() throws IOException{
        boolean error = false;
        if(!mivista.getElevacion().equals("") && !mivista.getElevacion().equals("Introduce el factor de elevacion")){
           FuenteDeDatos.setElevacion(mivista.getElevacion()); 
        }else{
            mivista.errorElevacion();
            error = true;
        }
        
        if(maquinaEstados.getModelo().getVisualizaciones().isEmpty()){
            error = true;
            mivista.errorVisualizaciones();
        }
        
        if(mivista.getRutaOrigen().equals("") || mivista.getRutaOrigen().equals("Introduce ruta de origen")){
            error = true;
            mivista.errorRutaOrigen();
        }
        
        if(mivista.getRutaR().equals("") || mivista.getRutaR().equals("Introduce la ruta de R")){
            error = true;
            mivista.errorR();
        }
        
        if(!mivista.getNombreD().equals("")){
            FuenteDeDatos.setNombreD(mivista.getNombreD());
        }
        
        if(!error){
            String ruta = "./temporales/principal.txt";
            File archivo = new File(ruta);
            BufferedWriter bw;
            if(!archivo.exists()) {
                bw = new BufferedWriter(new FileWriter(archivo));
                //bw.write(FuenteDeDatos.getrutaOrigen()+","+FuenteDeDatos.getSeparador()+","+FuenteDeDatos.getformatoDestino()+","+FuenteDeDatos.getrutaDestino());
                bw.write("1;2;3;4;5;6");
                bw.newLine();
                bw.write(FuenteDeDatos.getrutaOrigen()+";"+FuenteDeDatos.getSeparador()+";"+FuenteDeDatos.getformatoDestino()+";"+FuenteDeDatos.getElevacion()+";"+FuenteDeDatos.getNombreD()+";"+FuenteDeDatos.getrutaDestino());
                bw.newLine();
                if(!maquinaEstados.getModelo().getVisualizaciones().isEmpty()){
                    for(int i = 0; i<maquinaEstados.getModelo().getVisualizaciones().size(); i++){
                        Visualizacion vis = maquinaEstados.getModelo().getVisualizaciones().get(i);
                        bw.write(vis.getTipo()+";"+vis.getMedida()+";"+vis.getvrespuesta()+";"+vis.getvCruceUno()+";"+vis.getvCruceDos()+";nada");
                        bw.newLine();
                    }
                }
            } else {
                bw = new BufferedWriter(new FileWriter(archivo));
                //bw.write(FuenteDeDatos.getrutaOrigen()+","+FuenteDeDatos.getSeparador()+","+FuenteDeDatos.getformatoDestino()+","+FuenteDeDatos.getrutaDestino());
                bw.write("1;2;3;4;5;6");
                bw.newLine();
                bw.write(FuenteDeDatos.getrutaOrigen()+";"+FuenteDeDatos.getSeparador()+";"+FuenteDeDatos.getformatoDestino()+";"+FuenteDeDatos.getElevacion()+";"+FuenteDeDatos.getNombreD()+";"+FuenteDeDatos.getrutaDestino());
                bw.newLine();
                if(!maquinaEstados.getModelo().getVisualizaciones().isEmpty()){
                    for(int i = 0; i<maquinaEstados.getModelo().getVisualizaciones().size(); i++){
                        Visualizacion vis = maquinaEstados.getModelo().getVisualizaciones().get(i);
                        bw.write(vis.getTipo()+";"+vis.getMedida()+";"+vis.getvrespuesta()+";"+vis.getvCruceUno()+";"+vis.getvCruceDos()+";nada");
                        bw.newLine();
                    }
                }
            }
            bw.close();
            String rutaD = "./temporales/diccionario.txt";
            File dic = new File(rutaD);
            if(FuenteDeDatos.getExisteD()){
                bw = new BufferedWriter(new FileWriter(dic));
                bw.write("nombre_variable;codigo;descripcion");
                bw.newLine();
                if(!maquinaEstados.getModelo().getDiccionario().getNombres().isEmpty()){
                    Diccionario dicc = maquinaEstados.getModelo().getDiccionario();
                    for(int i = 0; i<dicc.getNombres().size(); i++){
                        bw.write(dicc.getNombres().get(i)+";"+dicc.getCodigos().get(i)+";"+dicc.getDescripciones().get(i));
                        bw.newLine();
                    }
                }
            }else{
                bw = new BufferedWriter(new FileWriter(dic));
                bw.write("nombre_variable;codigo;descripcion");
            }
            bw.close();
        }
        
        String rutaAux = FuenteDeDatos.getrutaDestino()+'\\'+FuenteDeDatos.getNombreD();
        String rutaFinal = rutaAux.replace("\\","\\\\");
        switch(FuenteDeDatos.getformatoDestino()){
            case "Excel":
                rutaCompleta = rutaFinal +".xlsx";
                break;
            case "PowerPoint": 
                rutaCompleta = rutaFinal +".pptx";
                break;
            case "Word":
                rutaCompleta = rutaFinal +".docx";
                break;
            case "Imagen":
                rutaCompleta = rutaFinal +".png";
                break;
            case "HTML":
                rutaCompleta = rutaFinal +".html";
                break;
        }
        File resultados = new File(rutaCompleta);
        if(resultados.exists()) {
           maquinaEstados.warningON();
        }else{
            String cmd = "\""+FuenteDeDatos.getRutaR()+"\" -e \"source('./tfg.r')\"";
            Runtime.getRuntime().exec(cmd);
            generando();
        }
        
    }
    
    /**
     * Metodo que le pide a la maquina de estados que cierre la ventana de alerta
     */
    public static void cancelarOperacion(){
        maquinaEstados.warningOFF();
    }
    
    /**
     * Metodo que le pide a la maquina de estados que muestre una ventana temporal
     * para informar al usuario de que se está generando el informe
     */
    public static void generando(){
        maquinaEstados.generando();
    }
    
    /**
     * Metodo que borra el archivo que ya estaba creado y llama a la aplicacion
     * de R para que cree el nuevo
     * @throws IOException excepcion de entrada/salida
     */
    public static void reemplazarArchivo() throws IOException{
        maquinaEstados.warningOFF();
        File antiguo = new File(rutaCompleta);
        antiguo.delete();
        String cmd = "\""+FuenteDeDatos.getRutaR()+"\" -e \"source('./tfg.r')\"";
        Runtime.getRuntime().exec(cmd);
        generando();
    }
    

    
}
