################################################################
################## TFG GRADO EN ESTADSTICA ####################
################################################################
###                RAUL HERNANSANZ QUEVEDO                   ###

#############################################
#--------------------------------------------
#Carga de las librerias necesarias para realizar graficos y tablas
#--------------------------------------------
install.packages("openxlsx",repos="https://cloud.r-project.org")
install.packages("R2HTML",repos="https://cloud.r-project.org")
install.packages("png",repos="https://cloud.r-project.org")
install.packages("officer",repos="https://cloud.r-project.org")
install.packages("gridExtra",repos="https://cloud.r-project.org")
install.packages("flextable",repos="https://cloud.r-project.org")

require(openxlsx)
require(png)
require(officer)
require(R2HTML)
require(flextable)
require(gridExtra)
#############################################

#############################################
#--------------------------------------------
#Esta funcion es la encargada de leer los datos. Se supondra que los archivos proporcionados tienen cabecera
#Parametros de entrada:
# -> ruta: ruta en la que se encuentra el archivo
# -> separador: separador de columnas del archivo de entrada
#Salida:
# -> datos: archivo de datos leido
#--------------------------------------------
lecturaDatos <- function (ruta, separador=','){
  a1=strsplit(basename(ruta),"[.]")[[1]]
  formato=toupper(a1[length(a1)])
  if(formato == "CSV"){
    datos <- read.csv(ruta, sep = separador)
    return (datos)  
  }
  if(formato == "TXT"){
    datos <- read.table(ruta,header=TRUE,sep=separador)
    return(datos)
  }
  if(formato == "TSV"){
    datos <- read.csv(ruta, sep = separador)
    return (datos)
  }
  if(formato == "XLSX"){
    datos <- read.xlsx(ruta, sheet = 1, colNames = TRUE)
    return (datos)
  }
}

#############################################

#############################################
#--------------------------------------------
#Esta funcion es la encargada de leer el diccionario.
#Parametros de entrada:
# -> rutad: ruta en la que se encuentra el archivo
#Salida:
# -> diccionario: archivo del diccionario leido
#--------------------------------------------
lecturaDiccionario <- function (rutad){
  diccionario<-read.table(rutad,sep=";",header=TRUE)
  return (diccionario)
}

#############################################


#############################################
#--------------------------------------------
# Funcion que crea las tablas solicitadas por el usuario
# Parametros de entrada:
# -> vars_cruce: variables de cruce
# -> var_calculo: variable respuesta
# -> factor: factor de elevacion
# -> tipo: tipo de medida representada
# -> diccionario: diccionario asociado
# Salida:
# -> tabla: tabla creada
#--------------------------------------------
crea_tablas<-function(vars_cruce,var_calculo,factor,tipo,diccionario){
  if(tipo=="MEDIA"){
    tabla_beta=tapply(datos[,factor]*datos[,var_calculo],datos[,vars_cruce],sum)/
      tapply(datos[,factor],datos[,vars_cruce],sum)
    tabla=cambia_nombres(tabla_beta,diccionario,vars_cruce)
  }
  if(tipo=="TOTAL"){
    tabla_beta=tapply(datos[,var_calculo],datos[,vars_cruce],sum)
    tabla=cambia_nombres(tabla_beta,diccionario,vars_cruce)
  }
  if(tipo=="PORCENTAJE"){
    tabla = tapply(datos[,factor]*datos[,var_calculo],datos[,vars_cruce],sum)/
      tapply(datos[,factor],datos[,vars_cruce],sum)
    if(length(vars_cruce)>1){
      tabla2 <- round(prop.table(tabla,1)*100,2)
    }else{
      tabla2 <- round(prop.table(tabla)*100,2)
    }
    tabla=cambia_nombres(tabla2,diccionario,vars_cruce)
  }
  if(length(dim(tabla))==1){
    n1=names(tabla)
    dim(tabla)=c(1,length(tabla))
    colnames(tabla)=n1
  }
  return(tabla)
}


#############################################


#############################################
#--------------------------------------------
#Esta funcion es la encargada de crear los graficos de barras tanto horizontales como verticales
#Parametros de entrada:
# -> vars_cruce: Par de variables que se desea cruzar
# -> var_calculo: Variable que se utilizara para el calculo
# -> factor: Factor de elevación del conjunto de datos
# -> tipo: Que se va a medir con el grafico. (medias, porcentajes)
# -> h: Indica si el grafico de barras sera horizontal o vertical
# -> diccionario: Diccionario util para esta encuesta que contiene el nombre de las variables, el codigo y la descripcion
# -> indiceG: indice del grafico actual necesario para darle nombre a la imagen creada
#Salida:
# -> Imagen: Imagen png con el grafico creado
#--------------------------------------------
crea_barras<-function(vars_cruce,var_calculo,factor,tipo,h,diccionario,indiceG){
  nombreA<-paste("auxG",indiceG,sep="_")
  nombreA<-paste(nombreA,"png",sep=".")
  nombreA<-paste("./resultados",nombreA,sep="/")
  if(tipo == "MEDIA"){
    tabla_beta=tapply(datos[,factor]*datos[,var_calculo],datos[,vars_cruce],sum)/
                                      tapply(datos[,factor],datos[,vars_cruce],sum)
    tabla=cambia_nombres(tabla_beta,diccionario,vars_cruce)
    if(length(vars_cruce)>1){
      n<-ncol(tabla)
    }else{
      n<-nrow(tabla)
    }
    png(nombreA)
    a1=barplot(t(tabla),col=rainbow(n),beside=TRUE,horiz=h,las=1, xpd=NA)
    if(length(vars_cruce)>1){
      legend(par("usr")[1],par("usr")[3]-strheight("A")*3,
                                      legend=colnames(tabla),fill=rainbow(n),xpd=NA,cex=0.6)
      abline(v=0)
    }
    
    text(c(t(tabla))/1.5,c(a1),formatC(c(t(tabla)),format="f",
                                       digits=2,decimal.mark=",",big.mark="."),col="white")
    dev.off()
  }
  if(tipo == "TOTAL"){
    tabla_beta=tapply(datos[,var_calculo],datos[,vars_cruce],sum)
    tabla=cambia_nombres(tabla_beta,diccionario,vars_cruce)
    if(length(vars_cruce)>1){
      n<-ncol(tabla)
    }else{
      n<-nrow(tabla)
    }
    png(nombreA)
    a1=barplot(t(tabla),col=rainbow(n),beside=TRUE,horiz=h,las=1, xpd=NA)
    if(length(vars_cruce)>1){
      legend(par("usr")[1],par("usr")[3]-strheight("A")*3,
                                       legend=colnames(tabla),fill=rainbow(n),xpd=NA,cex=0.6)
      abline(v=0)
    }
    text(c(t(tabla))/1.5,c(a1),formatC(c(t(tabla)),format="f",
                                       digits=2,decimal.mark=",",big.mark="."),col="white")
    dev.off()
  }
  if(tipo == "PORCENTAJE"){
    tabla = tapply(datos[,factor]*datos[,var_calculo],datos[,vars_cruce],sum)/
                                       tapply(datos[,factor],datos[,vars_cruce],sum)
    if(length(vars_cruce)>1){
    tabla2 <- round(prop.table(tabla,1)*100,2)
    }else{
      tabla2 <- round(prop.table(tabla)*100,2)
    }
    tabla=cambia_nombres(tabla2,diccionario,vars_cruce)
    if(length(vars_cruce)>1){
      n<-ncol(tabla)
    }else{
      n<-nrow(tabla)
    }
    png(nombreA)
    a1=barplot(t(tabla),col=rainbow(n),beside=TRUE,horiz=h,las=1)
    if(length(vars_cruce)>1){
      legend(par("usr")[1],par("usr")[3]-strheight("A")*3,
                                       legend=colnames(tabla),fill=rainbow(n),xpd=NA,cex=0.6)
      abline(v=0)
    }
    text(c(t(tabla))/1.5,c(a1),formatC(c(t(tabla)),format="f",
                                       digits=2,decimal.mark=",",big.mark="."),col="black")
    dev.off()
  }
}
#############################################

#############################################
#--------------------------------------------
#Esta funcion es la encargada de crear los graficos de sectores
#Parametros de entrada:
# -> vars_cruce: Par de variables que se desea cruzar
# -> var_calculo: Variable que se utilizara para el calculo
# -> factor: Factor de elevación del conjunto de datos
# -> tipo: Que se va a medir con el grafico. (medias, porcentajes)
# -> diccionario: Diccionario util para esta encuesta que contiene el nombre de las variables, el codigo y la descripcion
# -> indiceG: indice del grafico actual necesario para darle nombre a la imagen creada
#Salida:
# -> Prueba: Imagen png con el grafico creado
#--------------------------------------------
crea_pie<-function(vars_cruce,var_calculo,factor,tipo,diccionario,indiceG){
  nombreA<-paste("auxG",indiceG,sep="_")
  nombreA<-paste(nombreA,"png",sep=".")
  nombreA<-paste("./resultados",nombreA,sep="/")
  if(tipo == "MEDIA"){
    tabla_beta=tapply(datos[,factor]*datos[,var_calculo],datos[,vars_cruce],sum)/
      tapply(datos[,factor],datos[,vars_cruce],sum)
    tabla=cambia_nombres(tabla_beta,diccionario,vars_cruce)
    n<-nrow(tabla)
    png(nombreA)
    par(mar=c(6,6,6,6)) 
    pie(tabla, xpd=NA, cex= 0.8, radius = 1, col=rainbow(n),labels = round(as.numeric(tabla),2))
    legend(par("usr")[1],par("usr")[3]-strheight("A")*3,
           legend=rownames(tabla),fill=rainbow(n),horiz=FALSE,xpd=NA,cex=0.8)
    dev.off()
  }
  if(tipo == "PORCENTAJE"){
    tabla = tapply(datos[,factor]*datos[,var_calculo],datos[,vars_cruce],sum)/
      tapply(datos[,factor],datos[,vars_cruce],sum)
    tabla2 <- round(prop.table(tabla)*100,2)
    tabla=cambia_nombres(tabla2,diccionario,vars_cruce)
    n<-nrow(tabla)
    png(nombreA)
    par(mar=c(6,6,6,6)) 
    pie(tabla, xpd=NA, cex= 0.8, radius = 1, col=rainbow(n),labels = round(as.numeric(tabla),2))
    legend(par("usr")[1],par("usr")[3]-strheight("A")*3,
           legend=rownames(tabla),fill=rainbow(n),horiz=FALSE,xpd=NA,cex=0.8)
    dev.off()
  }
  if(tipo=="TOTAL"){
    tabla_beta=tapply(datos[,var_calculo],datos[,vars_cruce],sum)
    tabla=cambia_nombres(tabla_beta,diccionario,vars_cruce)
    n<-nrow(tabla)
    png(nombreA)
    par(mar=c(6,6,6,6)) 
    pie(tabla, xpd=NA, cex= 0.8, radius = 1, col=rainbow(n),labels = round(as.numeric(tabla),2))
    legend(par("usr")[1],par("usr")[3]-strheight("A")*3,
           legend=rownames(tabla),fill=rainbow(n),horiz=FALSE,xpd=NA,cex=0.8)
    dev.off()
  }
}
#############################################


#############################################
#---------------------------------------------
#Funcion que modifica el nombre de las columnas y de las filas de una tabla para adecuarlas a la descripcion
#proporcionada por el diccionario
#Parámetros de entrada:
# -> tabla: tabla a la que se van a cambiar los nombres de filas y columnas
# -> diccionario: diccionario de esta encuesta
# -> vars_cruce: variables utilizadas para la creacion de la tabla
#Parametros de salida:
# -> tabla: misma tabla de entrada con los nombres de filas y columnas actualizados
#---------------------------------------------
cambia_nombres<-function(tabla,diccionario,vars_cruce){
  if(length(vars_cruce)>1){
    row=diccionario[diccionario$nombre_variable==vars_cruce[1],]
    rownames(row)=row[,"codigo"]
    rownames(tabla)=row[rownames(tabla),"descripcion"]
    
    col=diccionario[diccionario$nombre_variable==vars_cruce[2],]
    rownames(col)=col[,"codigo"]
    colnames(tabla)=col[colnames(tabla),"descripcion"]
    return(tabla)
  }else{
    row=diccionario[diccionario$nombre_variable==vars_cruce[1],]
    rownames(row)=row[,"codigo"]
    rownames(tabla)=row[rownames(tabla),"descripcion"]
    return(tabla)
  }
}  
#############################################

#############################################
#---------------------------------------------
#Funcion que lee el archivo de parametros creada por la interfaz
#Parametros de salida:
# -> parametros: parametros que utilizara la aplicacion, asi como las visualizaciones que desea el usuario
#---------------------------------------------
lectura_parametros<-function(){
  parametros<- read.table("./temporales/principal.txt", sep =";", header=TRUE, encoding = "UTF-8")
  return(parametros)
}  
#############################################

#############################################
#---------------------------------------------
# Funcion que se encarga de crear el informe solicitado por el cliente
# Parametros de entrada:
# -> formatoS: formato en el que desea el informe el usuario
# -> nombreD: nombre que desea darle el usuario al informe
# -> rutaD: ruta en la que desea almacenar el informe el usuario
# -> indiceG: cantidad de graficos creados
# -> indiceT: cantidad de tablas creadas
# -> tablas: lista de tablas creadas
# Salida:
# Informe creado
#---------------------------------------------
creacionInforme<-function(formatoS,nombreD,rutaD,indiceG,indiceT,tablas){
  if(formatoS=="PowerPoint"){
    doc<-read_pptx()
    if(indiceG>0){
      for(i in 1:indiceG-1){
        
        nombre<-paste("auxG",i,sep="_")
        nombreF<-paste(nombre,"png",sep=".")
        ruta<-paste("./resultados",nombreF,sep="/")
        add_slide(doc,layout = "Title and Content", master="Office Theme") 
        doc <- on_slide(doc, index = i+1)
        img.file <- file.path(ruta )
        ph_with(x = doc, external_img(img.file, 1, 1),location = ph_location_type(type = "body"))
      }
    }
    if(indiceT>0){
      for(i in 1:indiceT){
        add_slide(doc,layout = "Title and Content", master="Office Theme")
        doc <- on_slide(doc, index = i+indiceG)
        if(!is.null(rownames(tablas[[i]]))){
          t1<-data.frame(" "=rownames(tablas[[i]]),tablas[[i]],check.names = FALSE)
        }else{
          t1=data.frame(tablas[[i]])
        }
        doc<-ph_with_table(doc,t1,location = ph_location_type(type = "body"))
      }
    }
    nombreSalida<-paste(nombreD,"pptx",sep=".")
    final<-paste(rutaD,nombreSalida,sep="/")
    print(doc,final)
  }
  if(formatoS == "Word"){
    doc<-read_docx()
    if(indiceG>0){
      for(i in 1:indiceG-1){
        nombre<-paste("auxG",i,sep="_")
        nombreF<-paste(nombre,"png",sep=".")
        ruta<-paste("./resultados",nombreF,sep="/")
        doc<-body_add_img(doc,ruta,width=5,height=5)
      }
    }
    
    if(indiceT>0){
      for(i in 1:indiceT){
        if(!is.null(rownames(tablas[[i]]))){
          t1<-data.frame(" "=rownames(tablas[[i]]),tablas[[i]],check.names = FALSE)
        }else{
          t1=data.frame(tablas[[i]])
        }
        doc<-body_add_table(doc,t1)
      }
    }
    nombreSalida<-paste(nombreD,"docx",sep=".")
    final<-paste(rutaD,nombreSalida,sep="/")
    print(doc,final)
  }
  if(formatoS=="Excel"){
    wb <- createWorkbook()
    if(indiceG>0){
      for(i in 1:indiceG-1){
        nombre<-paste("auxG",i,sep="_")
        nombreF<-paste(nombre,"png",sep=".")
        ruta<-paste("./resultados",nombreF,sep="/")
        addWorksheet(wb, nombre)
        insertImage(wb, nombre, ruta,width = 6, height = 3, startRow = 1,
                    startCol = 1, units = "in", dpi = 300)}
    }
    if(indiceT>0){
      for(i in 1:indiceT){
        name<-paste("Tabla",i,sep="_")
        addWorksheet(wb, name)
        writeData(wb, name, data.frame(tablas[[i]]),rowNames = TRUE)}
      }
    nombreSalida<-paste(nombreD,"xlsx",sep=".")
    final<-paste(rutaD,nombreSalida,sep="/")
    saveWorkbook(wb, file = final, overwrite = TRUE)
  }
  if(formatoS=="Imagen"){
    if(indiceG>0){
      listafi<-dir("./resultados",full.names = TRUE)
      file.copy(listafi, rutaD)
      file.remove(listafi)
    }
    if(indiceT>0){
      for(i in 1:indiceT-1){
        aux<-paste(nombreD,i,sep="_")
        nombreSalida<-paste(aux,"png",sep=".")
        final<-paste(rutaD,nombreSalida,sep="/")
        tabla<-as.data.frame(tablas[i+1])
        nr<-nrow(tabla)
        if(length(tabla)>1){
          nc<-ncol(tabla)
        }else{
          nc<-1
        }
        png(final, height = 50*nr, width = 200*nc)
        grid.table(tabla)
        dev.off()
      }
    }
  }

  if(formatoS=="HTML"){
    rutaF<-paste(rutaD,"resultadosHTML",sep="/")
    dir.create(rutaF,showWarnings = FALSE)
    HTMLStart(outdir=rutaF,filename=nombreD, extension="html")
    if(indiceG>0){
      for(i in 1:indiceG-1){
        nombre<-paste("auxG",0,sep="_")
        nombreF<-paste(nombre,"png",sep=".")
        ruta<-paste("./resultados",nombreF,sep="/")
        img<-readPNG(ruta)
        plot(1:2, type='n',ann=FALSE,axes = FALSE)
        rasterImage(img, 0.95, 0.95, 2.05, 2.05, interpolate=FALSE)
        HTMLplot()
      }
    }

    if(indiceT>0){
      for(i in 1:indiceT-1){
        HTML(tablas[i+1])
      }
    }
    HTMLStop()
  }
}  
#############################################


#############################################
#--------------------------------------------
#Obtencion de los parametros iniciales.
#--------------------------------------------
parametros<-lectura_parametros()
#------
ruta<-gsub("\\\\","/",as.character(parametros[1,1]))
#------
sep <- as.character(parametros[1,2])
separador<-switch(sep,"punto y coma"=";","tabulador" = "\t" , "coma"=",", "espacio"= " ")
#------
formatoS<-as.character(parametros[1,3])
#------
factor <- as.character(parametros[1,4])
#------
nombreD <- as.character(parametros[1,5])
#------
rutaD<-gsub("\\\\","/",parametros[1,6])
#------

datos <- lecturaDatos(ruta,separador)
diccionario <- lecturaDiccionario("./temporales/diccionario.txt")
attach(datos)

listaf<-dir("./resultados",full.names = TRUE)
file.remove(listaf)

nvis<-length(parametros[,1])-1
indiceG <- 0
indiceT <- 0
tablas=list()

for(i in 1:nvis){
  visualizacion<-as.character(parametros[i+1,1])
  medida <- toupper(as.character(parametros[i+1,2]))
  var_calculo<- as.character(parametros[i+1,3])
  vars_cruce<-c(as.character(parametros[i+1,4]))
  if(as.character(parametros[i+1,5])!="-"){
    vars_cruce<-c(vars_cruce,as.character(parametros[i+1,5]))
  }
  if(visualizacion == "Grafico de barras horizontales"){
    h = TRUE
    crea_barras(vars_cruce,var_calculo,factor,medida,h,diccionario,indiceG)
    indiceG<-indiceG+1
  }
  if(visualizacion == "Grafico de barras verticales"){
    h = FALSE
    crea_barras(vars_cruce,var_calculo,factor,medida,h,diccionario,indiceG)
    indiceG<-indiceG+1
  }
  if(visualizacion == "Grafico de sectores"){
    crea_pie(vars_cruce,var_calculo,factor,medida,diccionario,indiceG)
    indiceG<-indiceG+1
  }
  if(visualizacion == "Tabla"){
    tabla<-crea_tablas(vars_cruce,var_calculo,factor,medida,diccionario)
    tablas<-c(tablas,list(tabla))
    indiceT<-indiceT+1
  }
}

creacionInforme(formatoS,nombreD,rutaD,indiceG,indiceT,tablas)
creacionInforme(formatoS,nombreD,rutaD,indiceG,indiceT,tablas)
#############################################

